<?php

/**
 * @file
 * PxPay administration menu items.
 * @author Robbie Mackay <rm@robbiemackay.com>
 * @copyright Robbie MacKay 2009
 */

// Handles a completed PxPay sale.
function uc_dps_pxpay_complete($order_id = 0) {
  if (intval($_SESSION['cart_order']) != $order_id) {
    $_SESSION['cart_order'] = $order_id;
  }

  if (!($order = uc_order_load($order_id))) {
    drupal_goto('cart');
  }

  include('PxPay_Curl.inc');

  $pxpay = new PxPay_Curl(
    variable_get('uc_dps_pxpay_server', 'https://sec.paymentexpress.com/pxpay/pxaccess.aspx'),
    variable_get('uc_dps_pxpay_userid', ''),
    variable_get('uc_dps_pxpay_key', '')
  );

  $enc_hex = $_GET["result"];
  // getResponse method in PxPay object returns PxPayResponse object
  // which encapsulates all the response data
  $rsp = $pxpay->getResponse($enc_hex);

  if ($rsp->getSuccess() == "1") {
    // the following are the fields available in the PxPayResponse object
    $order->data['payment_details']['pxpay_Success'] = $rsp->getSuccess();   # =1 when request succeeds
    $payment_amount = $order->data['payment_details']['pxpay_AmountSettlement']  = $rsp->getAmountSettlement();
    $order->data['payment_details']['pxpay_AuthCode'] = $rsp->getAuthCode();  # from bank
    $order->data['payment_details']['pxpay_CardName'] = $rsp->getCardName();  # eg "Visa"
    $order->data['payment_details']['pxpay_CardNumber'] = $rsp->getCardNumber(); # Truncated card number
    $order->data['payment_details']['pxpay_CardHolderName'] = $rsp->getCardHolderName();

    $order->data['payment_details']['pxpay_DateExpiry'] = $rsp->getDateExpiry();
    $txn_id = $rsp->getDpsTxnRef();
    $order->data['payment_details']['pxpay_TxnType'] = $rsp->getTxnType();
    $order->data['payment_details']['pxpay_ClientInfo'] = $rsp->getClientInfo(); # The IP address of the user who submitted the transaction
    $order->data['payment_details']['pxpay_TxnId'] = $rsp->getTxnId();
    $order->data['payment_details']['pxpay_EmailAddress'] = $rsp->getEmailAddress();
    $order->data['payment_details']['pxpay_MerchantReference'] = $rsp->getMerchantReference();
    $order->data['payment_details']['pxpay_ResponseText'] = $rsp->getResponseText();

    $merchant_reference = variable_get('uc_dps_pxpay_refprefix', 'Website Order') ." #". $order->order_id;

    // Check merchant_ref, txn & email match.
    if ($order->data['pxpay_txnid'] != $rsp->getTxnId() ||
        $merchant_reference != $rsp->getMerchantReference() ||
        $rsp->getEmailAddress() != $order->primary_email) {
      watchdog('uc_cart', 'DPS PxPay details dont match order details! Cart order ID: @cart_order', array('@cart_order' => $order->order_id), WATCHDOG_NOTICE);

      uc_order_save($order);
      uc_order_update_status($order->order_id, 'pxpay_onhold');
      uc_order_comment_save($order->order_id, 0, t('DPS PxPay payment details do not match order.'), 'admin', 'pxpay_onhold');
      uc_order_comment_save($order->order_id, 0, t('DPS PxPay payment error - contact us for assistance.'), 'order', 'pxpay_onhold');

      drupal_set_message(t('The details of your DPS PxPay payment do not match your order. Please contact us for assistance. Reference Order Id: @orderid', array('@orderid' => $order->order_id)));
      drupal_goto(variable_get('uc_dps_pxpay_cancel_return_url', 'cart'));
      return;
    }
    
    // Check for duplicate transaction
    if ($txn_id != $order->data['payment_details']['pxpay_DpsTxnRef']) {
      $comment = t('PxPay transaction ref: @txn_id', array('@txn_id' => $txn_id));
      $payments = uc_payment_load_payments($order_id);
      $order->data['payment_details']['pxpay_DpsTxnRef'] = $txn_id;
      
      // Check for duplicate transaction
      $duplicate_flag = FALSE;
      foreach ($payments as $payment) {
        if ($payment->comment == $comment) {
          $duplicate_flag = TRUE;
        }
      }
      
      uc_order_save($order);
    
      // Move an order's status from "In Checkout" to "Payment Received"
      $status = db_result(db_query("SELECT order_status FROM {uc_orders} WHERE order_id = %d", $order->order_id));
      if (uc_order_status_data($status, 'state') == 'in_checkout') {
        uc_order_update_status($order->order_id, 'payment_received');
      }
      
      if (!$duplicate_flag) {
        uc_payment_enter($order_id, 'uc_dps_pxpay_dps', $payment_amount, $order->uid, $order->data['payment_details'], $comment);
        uc_order_comment_save($order->order_id, 0, t('DPS PxPay payment successful'), 'order', 'payment_received');
      }
    }
    /* Begin Duplicated code: just to be sure */
    uc_order_save($order);
    
    // Move an order's status from "In Checkout" to "Payment Received"
    $status = db_result(db_query("SELECT order_status FROM {uc_orders} WHERE order_id = %d", $order->order_id));
    if (uc_order_status_data($status, 'state') == 'in_checkout') {
      uc_order_update_status($order->order_id, 'payment_received');
    }
    /* End duplicated code */

    // This lets us know it's a legitimate access of the complete page.
    $_SESSION['do_complete'] = TRUE;
    drupal_set_message(t('Your DPS payment was successful.'));
    drupal_goto('cart/checkout/complete');
    return;
  }
  else {
    drupal_set_message(t('Your DPS payment was declined. Please feel free to continue shopping or contact us for assistance.'));
    drupal_goto(variable_get('uc_dps_pxpay_cancel_return_url', 'cart'));
    return;
  }
}

// Handles a canceled Website Payments Standard sale.
function uc_dps_pxpay_cancel() {
  unset($_SESSION['cart_order']);
  drupal_set_message(t('Your DPS payment was cancelled. Please feel free to continue shopping or contact us for assistance.'));
  drupal_goto(variable_get('uc_dps_pxpay_cancel_return_url', 'cart'));
}

